@@ -4,10 +4,10 @@ use log;
 use std::iter;
 use std::mem;
 
-use crate::language::FloatType;
+use crate::language::{IntType, FloatType};
 use crate::parser::stmt::{StmtMeta, Stmt, Label, StmtList, ControlFlow};
 use crate::parser::expr::{Expr, ExprMeta, ExprBlock, ConditionalBranch};
-use crate::parser::primary::{Atom, Primary};
+use crate::parser::primary::{Atom, Primary, AccessItem};
 use crate::parser::lvalue::{Assignment, Declaration, LValue, DeclType};
 use crate::parser::fundefs::{FunctionDef, SignatureDef, ParamDef, DefaultDef};
 use crate::runtime::vm::LocalIndex;
@@ -462,12 +462,12 @@ impl CodeGenerator<'_> {
     
     ///////// Constants /////////
     
-    fn get_or_make_const(&mut self, symbol: Option<&DebugSymbol>, value: Constant) -> CompileResult<ConstID> {
+    fn get_or_make_const(&mut self, value: Constant) -> CompileResult<ConstID> {
         self.builder_mut().get_or_insert_const(value)
     }
     
     fn emit_load_const(&mut self, symbol: Option<&DebugSymbol>, value: Constant) -> CompileResult<()> {
-        let cid = self.get_or_make_const(symbol, value)?;
+        let cid = self.get_or_make_const(value)?;
         
         if cid <= u8::MAX.into() {
             self.emit_instr_byte(symbol, OpCode::LoadConst, u8::try_from(cid).unwrap());
@@ -591,6 +591,12 @@ impl CodeGenerator<'_> {
     
     ///////// Statements /////////
     
+    fn compile_stmt_with_symbol(&mut self, stmt: &StmtMeta) -> CompileResult<()> {
+        let symbol = stmt.debug_symbol();
+        self.compile_stmt(Some(symbol), stmt.variant())
+            .map_err(|err| err.with_symbol(*symbol))
+    }
+    
     fn compile_stmt(&mut self, symbol: Option<&DebugSymbol>, stmt: &Stmt) -> CompileResult<()> {
         match stmt {
             Stmt::Loop { label, body } => self.compile_loop(symbol, label.as_ref(), body)?,
@@ -621,9 +627,7 @@ impl CodeGenerator<'_> {
     fn compile_stmt_list(&mut self, stmt_list: &StmtList) -> CompileResult<()> {
         // compile stmt suite
         for stmt in stmt_list.iter() {
-            let inner_symbol = stmt.debug_symbol();
-            self.compile_stmt(Some(inner_symbol), stmt.variant())
-                .map_err(|err| err.with_symbol(*inner_symbol))?;
+            self.compile_stmt_with_symbol(stmt)?;
         }
         
         // handle control flow
@@ -683,11 +687,17 @@ impl CodeGenerator<'_> {
     
     ///////// Expressions /////////
     
+    fn compile_expr_with_symbol(&mut self, expr: &ExprMeta) -> CompileResult<()> {
+        let symbol = expr.debug_symbol();
+        self.compile_expr(Some(symbol), expr.variant())
+            .map_err(|err| err.with_symbol(*symbol))
+    }
+    
     fn compile_expr(&mut self, symbol: Option<&DebugSymbol>, expr: &Expr) -> CompileResult<()> {
         match expr {
             Expr::Atom(atom) => self.compile_atom(symbol, atom)?,
             
-            Expr::Primary(primary) => unimplemented!(),
+            Expr::Primary(primary) => self.compile_primary(symbol, primary)?,
             
             Expr::UnaryOp(op, expr) => {
                 self.compile_expr(symbol, expr)?;
@@ -719,9 +729,7 @@ impl CodeGenerator<'_> {
             .map_err(|_| CompileError::from(ErrorKind::TupleLengthLimit))?;
         
         for expr in expr_list.iter() {
-            let inner_symbol = expr.debug_symbol();
-            self.compile_expr(Some(inner_symbol), expr.variant())
-                .map_err(|err| err.with_symbol(*inner_symbol))?;
+            self.compile_expr_with_symbol(expr)?;
         }
         
         self.emit_instr_byte(symbol, OpCode::Tuple, len);
@@ -735,15 +743,7 @@ impl CodeGenerator<'_> {
             Atom::BooleanLiteral(true) => self.emit_instr(symbol, OpCode::True),
             Atom::BooleanLiteral(false) => self.emit_instr(symbol, OpCode::False),
             
-            Atom::IntegerLiteral(value) => {
-                if let Ok(value) = u8::try_from(*value) {
-                    self.emit_instr_byte(symbol, OpCode::UInt8, value);
-                } else if let Ok(value) = i8::try_from(*value) {
-                    self.emit_instr_byte(symbol, OpCode::Int8, value.to_le_bytes()[0]);
-                } else {
-                    self.emit_load_const(symbol, Constant::from(*value))?;
-                }
-            },
+            Atom::IntegerLiteral(value) => self.compile_integer(symbol, *value)?,
             
             Atom::FloatLiteral(value) => {
                 if FloatType::from(i8::MIN) <= *value && *value <= FloatType::from(i8::MAX) {
@@ -765,6 +765,27 @@ impl CodeGenerator<'_> {
         Ok(())
     }
     
+    fn compile_integer(&mut self, symbol: Option<&DebugSymbol>, value: IntType) -> CompileResult<()> {
+        if let Ok(value) = u8::try_from(value) {
+            self.emit_instr_byte(symbol, OpCode::UInt8, value);
+        } else if let Ok(value) = i8::try_from(value) {
+            self.emit_instr_byte(symbol, OpCode::Int8, value.to_le_bytes()[0]);
+        } else {
+            self.emit_load_const(symbol, Constant::from(value))?;
+        }
+        Ok(())
+    }
+    
+    fn compile_float(&mut self, symbol: Option<&DebugSymbol>, value: FloatType) -> CompileResult<()> {
+        if FloatType::from(i8::MIN) <= value && value <= FloatType::from(i8::MAX) {
+            let value = value as i8;
+            self.emit_instr_byte(symbol, OpCode::Float8, value.to_le_bytes()[0]);
+        } else {
+            self.emit_load_const(symbol, Constant::from(value))?;
+        }
+        Ok(())
+    }
+    
     fn compile_name_lookup(&mut self, symbol: Option<&DebugSymbol>, name: &InternSymbol) -> CompileResult<()> {
         
         // Try loading a Local variable
@@ -779,7 +800,51 @@ impl CodeGenerator<'_> {
     }
     
     fn compile_primary(&mut self, symbol: Option<&DebugSymbol>, primary: &Primary) -> CompileResult<()> {
-        unimplemented!()
+        self.compile_atom(symbol, primary.atom())?;
+        
+        for item in primary.path().iter() {
+            match item {
+                AccessItem::Attribute(name) => unimplemented!(),
+                AccessItem::Index(index) => unimplemented!(),
+                AccessItem::Invoke { args, unpack } => self.compile_invocation(symbol, args, unpack.as_ref())?,
+            }
+        }
+        
+        Ok(())
+    }
+    
+    fn compile_invocation(&mut self, symbol: Option<&DebugSymbol>, args: &[ExprMeta], unpack: Option<&ExprMeta>) -> CompileResult<()> {
+        // prepare argument list:
+        // [ callobj arg[n] arg[0] ... arg[n-1] nargs ] => [ ret_value ] 
+
+        let arg_len;
+        
+        if let Some((arg_last, args)) = args.split_last() {
+            
+            self.compile_expr_with_symbol(arg_last)?;
+            
+            for arg_expr in args.iter() {
+                self.compile_expr_with_symbol(arg_expr)?;
+            }
+            
+            arg_len = u8::try_from(args.len() + 1)
+                .map_err(|_| CompileError::from(ErrorKind::ArgCountLimit))?;
+            
+        } else {
+            
+            arg_len = 0;
+        }
+        
+        if let Some(seq_expr) = unpack {
+            self.compile_expr_with_symbol(seq_expr)?;
+            self.emit_instr_byte(symbol, OpCode::UInt8, arg_len);
+            self.emit_instr(symbol, OpCode::CallUnpack);
+        } else {
+            self.emit_instr_byte(symbol, OpCode::UInt8, arg_len);
+            self.emit_instr(symbol, OpCode::Call);
+        }
+
+        Ok(())
     }
     
     fn emit_unary_op(&mut self, symbol: Option<&DebugSymbol>, op: &UnaryOp) {
@@ -1020,9 +1085,7 @@ impl CodeGenerator<'_> {
         
         // result expression
         if let Some(expr) = suite.result() {
-            let inner_symbol = expr.debug_symbol();
-            self.compile_expr(Some(inner_symbol), expr.variant())
-                .map_err(|err| err.with_symbol(*inner_symbol))?;
+            self.compile_expr_with_symbol(expr)?;
         } else {
             self.emit_instr(symbol, OpCode::Nil); // implicit nil
         }
@@ -1096,6 +1159,7 @@ impl CodeGenerator<'_> {
         Ok(())
     }
     
+    
     ///////// Function Definitions /////////
     
     fn compile_function_def(&mut self, symbol: Option<&DebugSymbol>, fundef: &FunctionDef) -> CompileResult<()> {
@@ -1104,8 +1168,14 @@ impl CodeGenerator<'_> {
         let chunk_id = chunk.chunk_id().unwrap();
         
         // and a new local scope
-        chunk.emit_begin_scope(ScopeTag::Function, symbol);
+        // don't need to emit new scope instructions, should handled by function call
+        chunk.scope_mut().push_scope(ScopeTag::Function, symbol);
         
+        // don't need to generate IN_LOCAL instructions for these, the VM should include them automatically
+        chunk.scope_mut().insert_local(DeclType::Immutable, LocalName::Receiver)?;
+        chunk.scope_mut().insert_local(DeclType::Immutable, LocalName::NArgs)?;
+        
+        // prepare argument list
         chunk.compile_function_preamble(symbol, fundef)?;
         
         // function body
@@ -1115,13 +1185,14 @@ impl CodeGenerator<'_> {
         
         // function result
         if let Some(expr) = fundef.body.result() {
-            chunk.compile_expr(Some(expr.debug_symbol()), expr.variant())?;
+            chunk.compile_expr_with_symbol(expr)?;
         } else {
             chunk.emit_instr(symbol, OpCode::Nil);
         }
         
         // end the function scope
-        chunk.emit_end_scope();
+        // don't need to emit end scope instructions, should be handled by return
+        chunk.scope_mut().pop_scope();
         chunk.finish();
         
         // compile the function signature
@@ -1139,12 +1210,6 @@ impl CodeGenerator<'_> {
         // define locals
         let signature = &fundef.signature;
         
-        self.scope_mut().insert_local(DeclType::Immutable, LocalName::Receiver)?;
-        self.emit_instr(None, OpCode::InsertLocal);
-        
-        self.scope_mut().insert_local(DeclType::Immutable, LocalName::NArgs)?;
-        self.emit_instr(None, OpCode::InsertLocal);
-        
         for param in signature.required.iter() {
             self.scope_mut().insert_local(param.decl, LocalName::Symbol(param.name))?;
             self.emit_instr(None, OpCode::InsertLocal);
@@ -1224,8 +1289,8 @@ impl CodeGenerator<'_> {
             self.compile_expr(symbol, expr)?;
         }
         
-        self.emit_instr(None, OpCode::Pop);  // drop "defaults passed"
         jump_targets.insert(default_count.into(), self.current_offset());
+        self.emit_instr(None, OpCode::Pop);  // drop "defaults passed"
         
         // patch all jumps
         for idx in 0..=default_count.into() {
@@ -1272,26 +1337,30 @@ impl CodeGenerator<'_> {
     }
     
     fn compile_function_signature(&mut self, symbol: Option<&DebugSymbol>, signature: &SignatureDef) -> CompileResult<Signature> {
+        let name = 
+            if let Some(name) = signature.name {
+                Some(self.get_or_make_const(Constant::from(name))?)
+            } else { None };
+        
         let mut required = Vec::new();
         for param in signature.required.iter() {
-            let name = self.get_or_make_const(symbol, Constant::from(param.name))?;
+            let name = self.get_or_make_const(Constant::from(param.name))?;
             required.push(Parameter::new(name, param.decl));
         }
         
         let mut default = Vec::new();
         for param in signature.default.iter() {
-            let name = self.get_or_make_const(symbol, Constant::from(param.name))?;
+            let name = self.get_or_make_const(Constant::from(param.name))?;
             default.push(Parameter::new(name, param.decl));
         }
         
         let mut variadic = None;
         if let Some(param) = &signature.variadic {
-            let name = self.get_or_make_const(symbol, Constant::from(param.name))?;
+            let name = self.get_or_make_const(Constant::from(param.name))?;
             variadic.replace(Parameter::new(name, param.decl));
         }
         
-        Ok(Signature::new(required, default, variadic))
+        Ok(Signature::new(name, required, default, variadic))
     }
     
-
 }
