@@ -12,9 +12,15 @@ use std::mem::size_of;
 const OP_NOP:           u8 = 0x00;
 const OP_RETURN:        u8 = 0x01;  // return from current function?
 
-// yes, Nargs is both in the immediate stack and encoded in the call instruction
-// this bit of redundancy helps us deal with default and variadic arguments efficiently
-const OP_CALL:          u8 = 0x02;  // (u8); [ callobj Nargs arg_0 ... arg_n ] => [ ret_value ] 
+// The odd argument order is because the VM will swap "nargs" with "arg[n]" after reading "nargs".
+// This allows the full argument list (including default and variadic arguments) to be prepared
+// in a way that can be reasoned about at compile-time.
+
+// [ callobj arg[n] arg[0] ... arg[n-1] nargs ] => [ ret_value ] 
+const OP_CALL:          u8 = 0x02;
+
+// [ callobj arg[n] arg[0] ... arg[n-1] arg_seq nargs ] => [ ret_value ]  -- nargs does not include arg_seq! 
+const OP_CALL_UNPACK:   u8 = 0x03;
 
 // 0x08-40        Immediate Values
 
@@ -130,6 +136,7 @@ pub enum OpCode {
     Nop = OP_NOP,
     Return = OP_RETURN, 
     Call = OP_CALL,
+    CallUnpack = OP_CALL_UNPACK,
     
     Pop = OP_POP,
     Drop = OP_DROP,
@@ -206,6 +213,7 @@ impl OpCode {
             OP_NOP => Self::Nop,
             OP_RETURN => Self::Return,
             OP_CALL => Self::Call,
+            OP_CALL_UNPACK => Self::CallUnpack,
             
             OP_POP => Self::Pop,
             OP_DROP => Self::Drop,
@@ -283,8 +291,7 @@ impl OpCode {
     pub const fn instr_len(&self) -> usize {
         match self {
             // don't really need size_of() for most of these, but it's a nice little bit of self-documentation
-            Self::Call           => 1 + size_of::<u8>(),
-            
+
             Self::Drop           => 1 + size_of::<u8>(),
             
             Self::LoadConst      => 1 + size_of::<u8>(),
@@ -324,73 +331,74 @@ impl PartialEq<u8> for OpCode {
 impl std::fmt::Display for OpCode {
     fn fmt(&self, fmt: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         let mnemonic = match *self {
-            Self::Nop => "OP_NOP",
-            Self::Return => "OP_RETURN",
-            Self::Call => "OP_CALL",
+            Self::Nop => "NOP",
+            Self::Return => "RETURN",
+            Self::Call => "CALL",
+            Self::CallUnpack => "CALL_UNPACK",
             
-            Self::Pop => "OP_POP",
-            Self::Drop => "OP_DROP",
-            Self::Clone => "OP_CLONE",
+            Self::Pop => "POP",
+            Self::Drop => "DROP",
+            Self::Clone => "CLONE",
             
-            Self::Tuple => "OP_TUPLE",
-            Self::TupleN => "OP_TUPLEN",
+            Self::Tuple => "TUPLE",
+            Self::TupleN => "TUPLEN",
             
-            Self::LoadConst => "OP_LD_CONST",
-            Self::LoadConst16 => "OP_LD_CONST_16",
+            Self::LoadConst => "LD_CONST",
+            Self::LoadConst16 => "LD_CONST_16",
             
-            Self::InsertGlobal => "OP_IN_GLOBAL_IM",
-            Self::InsertGlobalMut => "OP_IN_GLOBAL_MUT",
-            Self::StoreGlobal => "OP_ST_GLOBAL",
-            Self::LoadGlobal => "OP_LD_GLOBAL",
+            Self::InsertGlobal => "IN_GLOBAL_IM",
+            Self::InsertGlobalMut => "IN_GLOBAL_MUT",
+            Self::StoreGlobal => "ST_GLOBAL",
+            Self::LoadGlobal => "LD_GLOBAL",
             
-            Self::InsertLocal => "OP_IN_LOCAL",
-            Self::StoreLocal => "OP_ST_LOCAL",
-            Self::StoreLocal16 => "OP_ST_LOCAL_16",
-            Self::LoadLocal => "OP_LD_LOCAL",
-            Self::LoadLocal16 => "OP_LD_LOCAL_16",
-            Self::DropLocals => "OP_DP_LOCALS",
+            Self::InsertLocal => "IN_LOCAL",
+            Self::StoreLocal => "ST_LOCAL",
+            Self::StoreLocal16 => "ST_LOCAL_16",
+            Self::LoadLocal => "LD_LOCAL",
+            Self::LoadLocal16 => "LD_LOCAL_16",
+            Self::DropLocals => "DP_LOCALS",
             
-            Self::Nil => "OP_LD_NIL",
-            Self::True => "OP_LD_TRUE",
-            Self::False => "OP_LD_FALSE",
-            Self::Empty => "OP_LD_EMPTY",
-            Self::UInt8 => "OP_LD_U8",
-            Self::Int8 => "OP_LD_I8",
-            Self::Float8 => "OP_LD_F8",
+            Self::Nil => "LD_NIL",
+            Self::True => "LD_TRUE",
+            Self::False => "LD_FALSE",
+            Self::Empty => "LD_EMPTY",
+            Self::UInt8 => "LD_U8",
+            Self::Int8 => "LD_I8",
+            Self::Float8 => "LD_F8",
             
-            Self::Neg => "OP_NEG",
-            Self::Pos => "OP_POS",
-            Self::Inv => "OP_INV",
-            Self::Not => "OP_NOT",
+            Self::Neg => "NEG",
+            Self::Pos => "POS",
+            Self::Inv => "INV",
+            Self::Not => "NOT",
             
-            Self::And => "OP_AND",
-            Self::Xor => "OP_XOR",
-            Self::Or => "OP_OR",
-            Self::Shl => "OP_SHL",
-            Self::Shr => "OP_SHR",
-            Self::Add => "OP_ADD",
-            Self::Sub => "OP_SUB",
-            Self::Mul => "OP_MUL",
-            Self::Div => "OP_DIV",
-            Self::Mod => "OP_MOD",
-            Self::EQ => "OP_EQ",
-            Self::NE => "OP_NE",
-            Self::LT => "OP_LT",
-            Self::LE => "OP_LE",
-            Self::GE => "OP_GE",
-            Self::GT => "OP_GT",
+            Self::And => "AND",
+            Self::Xor => "XOR",
+            Self::Or =>  "OR",
+            Self::Shl => "SHL",
+            Self::Shr => "SHR",
+            Self::Add => "ADD",
+            Self::Sub => "SUB",
+            Self::Mul => "MUL",
+            Self::Div => "DIV",
+            Self::Mod => "MOD",
+            Self::EQ => "CMP_EQ",
+            Self::NE => "CMP_NE",
+            Self::LT => "CMP_LT",
+            Self::LE => "CMP_LE",
+            Self::GE => "CMP_GE",
+            Self::GT => "CMP_GT",
             
-            Self::Jump => "OP_JUMP",
-            Self::JumpIfFalse => "OP_JUMP_FALSE",
-            Self::JumpIfTrue => "OP_JUMP_TRUE",
-            Self::PopJumpIfFalse => "OP_PJMP_FALSE",
-            Self::PopJumpIfTrue => "OP_PJMP_TRUE",
+            Self::Jump => "JUMP",
+            Self::JumpIfFalse => "JUMP_FALSE",
+            Self::JumpIfTrue => "JUMP_TRUE",
+            Self::PopJumpIfFalse => "PJMP_FALSE",
+            Self::PopJumpIfTrue => "PJMP_TRUE",
             
-            Self::LongJump => "OP_LJUMP",
-            Self::LongJumpIfFalse => "OP_LJUMP_FALSE",
-            Self::LongJumpIfTrue => "OP_LJUMP_TRUE",
-            Self::PopLongJumpIfFalse => "OP_PLJMP_FALSE",
-            Self::PopLongJumpIfTrue => "OP_PLJMP_TRUE",
+            Self::LongJump => "LJUMP",
+            Self::LongJumpIfFalse => "LJUMP_FALSE",
+            Self::LongJumpIfTrue => "LJUMP_TRUE",
+            Self::PopLongJumpIfFalse => "PLJMP_FALSE",
+            Self::PopLongJumpIfTrue => "PLJMP_TRUE",
             
             Self::Inspect => "DBG_INSPECT",
             Self::Assert => "DBG_ASSERT",
