@@ -46,6 +46,7 @@ pub fn create_default_lexer_rules() -> LexerBuilder {
     .add_rule(SingleCharRule::new(Token::Comma,           ','))
     .add_rule(SingleCharRule::new(Token::Colon,           ':'))
     .add_rule(SingleCharRule::new(Token::Semicolon,       ';'))
+    .add_rule(SingleCharRule::new(Token::Decorator,       '@'))
     
     .add_rule(MultiCharRule::new(Token::Ellipsis,         "..."))
     
@@ -103,9 +104,10 @@ pub fn create_default_lexer_rules() -> LexerBuilder {
     .add_rule(KeywordRule::new(Token::Then,               "then"))
     .add_rule(KeywordRule::new(Token::Elif,               "elif"))
     .add_rule(KeywordRule::new(Token::Else,               "else"))
+    .add_rule(KeywordRule::new(Token::Loop,               "loop"))
+    .add_rule(KeywordRule::new(Token::While,              "while"))
     .add_rule(KeywordRule::new(Token::For,                "for"))
     .add_rule(KeywordRule::new(Token::In,                 "in"))
-    .add_rule(KeywordRule::new(Token::While,              "while"))
     .add_rule(KeywordRule::new(Token::Do,                 "do"))
     .add_rule(KeywordRule::new(Token::Continue,           "continue"))
     .add_rule(KeywordRule::new(Token::Break,              "break"))
