@@ -20,7 +20,7 @@ pub use errors::{ParserError, ParseResult};
 
 use expr::{ExprMeta, Expr, ExprBlock, ConditionalBranch};
 use stmt::{StmtMeta, StmtList, Stmt, Label, ControlFlow};
-use primary::{Primary, Atom, AccessItem, Argument};
+use primary::{Primary, Atom, AccessItem};
 use lvalue::{Assignment, LValue, Declaration, DeclType};
 use operator::{UnaryOp, BinaryOp, Precedence, PRECEDENCE_START, PRECEDENCE_END};
 use fundefs::{FunctionDef, SignatureDef, ParamDef, DefaultDef};
@@ -873,10 +873,15 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
             } else { None };
         
         
-        let function_def = self.parse_function_def(ctx)?;
+        let mut function_def = self.parse_function_def(ctx)?;
         
         // SYNTACTIC SUGAR: fun name(..) => let name = fun(..)
         if let Some(lvalue) = name_lvalue {
+            // record name in function signature
+            if let LValue::Identifier(name) = &lvalue {
+                function_def.signature.name.replace(*name);
+            }
+            
             let function_decl = Declaration {
                 decl: DeclType::Immutable,
                 lhs: lvalue,
@@ -1053,7 +1058,14 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
             ctx.pop_extend();
         }
         
-        Ok(SignatureDef::new(required, default, variadic))
+        let signature = SignatureDef {
+            name: None,
+            required: required.into_boxed_slice(),
+            default: default.into_boxed_slice(),
+            variadic,
+        };
+        
+        Ok(signature)
     }
     
     /*
@@ -1200,30 +1212,49 @@ impl<'h, I> Parser<'h, I> where I: Iterator<Item=Result<TokenMeta, LexerError>>
         debug_assert!(matches!(next.token, Token::OpenParen));
         
         let mut args = Vec::new();
+        let mut unpack = None;
         
-        loop {
-            if matches!(self.peek()?.token, Token::CloseParen) {
-                break;
+        // check for empty argument list
+        if !matches!(self.peek()?.token, Token::CloseParen) {
+        
+            // we use the fact that argument lists parse very similarly to tuples
+            // parse a single ExprMeta. If the variant is a Tuple (not a group containing a tuple!),
+            // that tuple's contents becomes the argument list.
+            
+            let (expr, symbol) = self.parse_expr(ctx)?.take();
+            
+            if let Expr::Tuple(arg_list) = expr {
+                args.extend(arg_list.into_vec().into_iter());
+            } else {
+                args.push(ExprMeta::new(expr, symbol));
             }
             
-            let arg_expr = self.parse_expr(ctx)?;
-            if let Token::Ellipsis = self.peek()?.token {
+            // check for unpack syntax
+            if matches!(self.peek()?.token, Token::Ellipsis) {
                 ctx.set_end(&self.advance().unwrap());
                 
-                args.push(Argument::unpack(arg_expr));
-            } else { 
-                args.push(Argument::new(arg_expr));
+                if let Some(expr) = args.pop() {
+                    unpack.replace(expr);
+                } else {
+                    return Err("no argument to unpack".into());
+                }
             }
             
-            match self.peek()?.token {
-                Token::Comma => ctx.set_end(&self.advance().unwrap()),
-                Token::CloseParen => break,
-                _ => return Err("missing \",\" between invocation arguments".into()),
+            // check for close paren
+            let next = self.advance()?;
+            ctx.set_end(&next);
+            if !matches!(next.token, Token::CloseParen) {
+                return Err("missing closing \")\" after argument list".into());
             }
         }
         
+        let invocation = AccessItem::Invoke {
+            args: args.into_boxed_slice(),
+            unpack,
+        };
+        
         ctx.pop_extend();
-        Ok(AccessItem::Invoke(args.into_boxed_slice()))
+        Ok(invocation)
     }
     
     // atom ::= LITERAL | IDENTIFIER | "(" expression ")" ;
