@@ -10,27 +10,14 @@ pub struct FunctionDef {
     pub body: Box<ExprBlock>,
 }
 
-
 #[derive(Debug, Clone)]
 pub struct SignatureDef {
+    pub name: Option<InternSymbol>,
     pub required: Box<[ParamDef]>,
     pub default: Box<[DefaultDef]>,
     pub variadic: Option<ParamDef>,
 }
 
-impl SignatureDef {
-    pub fn new(required: Vec<ParamDef>, default: Vec<DefaultDef>, variadic: Option<ParamDef>) -> Self {
-        SignatureDef {
-            required: required.into_boxed_slice(),
-            default: default.into_boxed_slice(),
-            variadic,
-        }
-    }
-    
-    pub fn is_variadic(&self) -> bool { self.variadic.is_some() }
-}
-
-
 #[derive(Debug, Clone)]
 pub struct ParamDef {
     pub name: InternSymbol,
