@@ -23,47 +23,35 @@ pub enum Atom {
 #[derive(Debug, Clone)]
 pub enum AccessItem {
     Attribute(InternSymbol),
+    
     Index(ExprMeta),
-    Invoke(Box<[Argument]>),
+    
+    Invoke { 
+        args: Box<[ExprMeta]>, 
+        unpack: Option<ExprMeta> 
+    },
+    
     // Construct(ObjectConstructor),
 }
 
 #[derive(Debug, Clone)]
 pub struct Primary {
     atom: Atom,
-    path: Vec<AccessItem>,
+    path: Box<[AccessItem]>,
 }
 
 impl Primary {
     pub fn new(atom: Atom, path: Vec<AccessItem>) -> Self {
-        debug_assert!(!path.is_empty());
-        Primary { atom, path }
+        Primary { atom, path: path.into_boxed_slice() }
+    }
+    
+    pub fn take(self) -> (Atom, Vec<AccessItem>) {
+        (self.atom, self.path.into_vec())
     }
     
     pub fn atom(&self) -> &Atom { &self.atom }
-    pub fn take_atom(self) -> Atom { self.atom }
     
-    pub fn path(&self) -> &Vec<AccessItem> { &self.path }
-    pub fn path_mut(&mut self) -> &mut Vec<AccessItem> { &mut self.path }
+    pub fn path(&self) -> &[AccessItem] { &self.path }
+    pub fn path_mut(&mut self) -> &mut [AccessItem] { &mut self.path }
 }
 
-
-#[derive(Debug, Clone)]
-pub struct Argument {
-    value: ExprMeta,
-    unpack: bool,
-}
-
-impl Argument {
-    pub fn new(value: ExprMeta) -> Self {
-        Self { value, unpack: false }
-    }
-    
-    pub fn unpack(value: ExprMeta) -> Self {
-        Self { value, unpack: true }
-    }
-    
-    pub fn is_unpack(&self) -> bool { self.unpack }
-    pub fn value(&self) -> &ExprMeta { &self.value }
-    
-}
