@@ -1,7 +1,7 @@
 use crate::parser::lvalue::DeclType;
 use crate::codegen::{ChunkID, ConstID};
 use crate::runtime::module::{ModuleID, Access};
-use crate::runtime::strings::StringSymbol;
+use crate::runtime::gc::GCObject;
 
 
 pub struct Function {
@@ -10,23 +10,41 @@ pub struct Function {
     chunk_id: ChunkID,
 }
 
+impl Function {
+    pub fn new(signature: Signature, module_id: ModuleID, chunk_id :ChunkID) -> Self {
+        Self { signature, module_id, chunk_id }
+    }
+}
+
+impl From<Function> for GCObject {
+    fn from(function: Function) -> Self {
+        Self::Function(Box::new(function))
+    }
+}
+
+
 #[derive(Clone, Debug)]
 pub struct Signature {
-    // all parameters are positional
+    name: Option<ConstID>,
     required: Box<[Parameter]>,
     default: Box<[Parameter]>,
     variadic: Option<Parameter>,
 }
 
 impl Signature {
-    pub fn new(required: Vec<Parameter>, default: Vec<Parameter>, variadic: Option<Parameter>) -> Self {
+    pub fn new(name: Option<ConstID>, required: Vec<Parameter>, default: Vec<Parameter>, variadic: Option<Parameter>) -> Self {
         Self {
+            name,
             required: required.into_boxed_slice(),
             default: default.into_boxed_slice(),
             variadic,
         }
     }
     
+    pub fn name(&self) -> Option<ConstID> {
+        self.name
+    }
+    
     pub fn is_variadic(&self) -> bool {
         self.variadic.is_some()
     }
