@@ -84,15 +84,15 @@ impl From<bool> for Variant {
 }
 
 impl From<IntType> for Variant {
-    fn from(value: IntType) -> Self { Variant::Integer(value) }
+    fn from(value: IntType) -> Self { Self::Integer(value) }
 }
 
 impl From<FloatType> for Variant {
-    fn from(value: FloatType) -> Self { Variant::Float(value) }
+    fn from(value: FloatType) -> Self { Self::Float(value) }
 }
 
 impl From<StringSymbol> for Variant {
-    fn from(value: StringSymbol) -> Self { Variant::String(value) }
+    fn from(value: StringSymbol) -> Self { Self::String(value) }
 }
 
 impl From<&str> for Variant {
@@ -102,6 +102,10 @@ impl From<&str> for Variant {
     }
 }
 
+impl From<GCHandle> for Variant {
+    fn from(handle: GCHandle) -> Self { Self::Object(handle) }
+}
+
 
 // Not all Variants are hashable, so there is a separate type to handle that
 
@@ -188,7 +192,7 @@ impl fmt::Debug for Variant {
                 write!(fmt, "{:?})", last)
             }
             
-            Self::Object(_handle) => unimplemented!(),
+            Self::Object(handle) => write!(fmt, "{:?}", handle),
         }
     }
 }
@@ -208,6 +212,8 @@ impl fmt::Display for Variant {
                 write!(fmt, "{})", last)
             }
             
+            // Self::Object(handle) => // TODO invoke __tostring
+            
             _ => write!(fmt, "{}", self)
         }
     }
