@@ -47,11 +47,7 @@ impl<'m> VirtualMachine<'m> {
         }
     }
     
-    pub fn run(&mut self) -> ExecResult<()> {
-        if self.states.is_empty() {
-            panic!("uninitialized vm")
-        }
-        
+    pub fn run(mut self) -> ExecResult<()> {
         while let Some(state) = self.states.last_mut() {
             match state.exec_next(&mut self.values)? {
                 Control::Continue => { }
@@ -85,6 +81,7 @@ impl ValueStack {
         self.stack.len()
     }
     
+    #[inline(always)]
     fn clear(&mut self) {
         self.stack.clear()
     }
@@ -125,6 +122,19 @@ impl ValueStack {
         *self.stack.last_mut().expect("empty stack") = value;
     }
     
+    #[inline(always)]
+    fn replace_many(&mut self, count: usize, value: Variant) {
+        let replace_range = (self.stack.len()-count).. ;
+        self.stack.splice(replace_range, std::iter::once(value));
+    }
+    
+    #[inline(always)]
+    fn replace_at(&mut self, index: usize, value: Variant) {
+        let item = self.stack.get_mut(index)
+            .expect("value index out of bounds");
+        *item = value;
+    }
+    
     #[inline(always)]
     fn peek(&self) -> &Variant {
         self.stack.last().expect("empty stack")
@@ -142,13 +152,6 @@ impl ValueStack {
         peek
     }
     
-    #[inline(always)]
-    fn replace_at(&mut self, index: usize, value: Variant) {
-        let item = self.stack.get_mut(index)
-            .expect("value index out of bounds");
-        
-        *item = value;
-    }
 }
 
 
@@ -258,6 +261,7 @@ impl<'m> VMState<'m> {
         panic!("invalid operand")
     }
 
+    #[inline(always)]
     fn exec_next(&mut self, stack: &mut ValueStack) -> ExecResult<Control> {
         let op_byte = self.chunk.get(self.pc).expect("pc out of bounds");
         let opcode = OpCode::from_byte(*op_byte)
@@ -273,6 +277,7 @@ impl<'m> VMState<'m> {
             OpCode::Return => return Ok(Control::Return),
             
             OpCode::Call => unimplemented!(),
+            OpCode::CallUnpack => unimplemented!(),
             
             OpCode::Pop => { 
                 stack.pop(); 
@@ -347,8 +352,12 @@ impl<'m> VMState<'m> {
             },
             OpCode::DropLocals => {
                 let count = LocalIndex::from(data[0]);
-                let index = usize::from(self.locals) - usize::from(count);
-                stack.discard_at(index, usize::from(count));
+                if stack.len() == self.locals.into() {
+                    stack.discard(usize::from(count));
+                } else {
+                    let index = usize::from(self.locals) - usize::from(count);
+                    stack.discard_at(index, usize::from(count));
+                }
                 self.locals -= count;
             },
             
@@ -359,14 +368,27 @@ impl<'m> VMState<'m> {
 
             OpCode::Tuple => {
                 let tuple_len = usize::from(data[0]);
-                let items = stack.pop_many(tuple_len).into_boxed_slice();
-                stack.push(Variant::make_tuple(items));
+                
+                // peek and clone to ensure items stay rooted for GC
+                let items = stack.peek_many(tuple_len).iter()
+                    .cloned().collect::<Vec<Variant>>()
+                    .into_boxed_slice();
+                
+                let tuple = Variant::make_tuple(items);
+                stack.replace_many(tuple_len, tuple);
             },
             OpCode::TupleN => {
                 let tuple_len = Self::into_usize(stack.pop());
-                let items = stack.pop_many(tuple_len).into_boxed_slice();
-                stack.push(Variant::make_tuple(items));
+                
+                // peek and clone to ensure items stay rooted for GC
+                let items = stack.peek_many(tuple_len).iter()
+                    .cloned().collect::<Vec<Variant>>()
+                    .into_boxed_slice();
+                
+                let tuple = Variant::make_tuple(items);
+                stack.replace_many(tuple_len, tuple);
             },
+            
             OpCode::UInt8 => {
                 let value = IntType::from(data[0]);
                 stack.push(Variant::Integer(value))
